function X=metrics3D(seg,truth)
% function X=metrics3D(SEG,TRUTH)
% SEG and TRUTH are binary lesion segmentation volumes
% SEG and TRUTH must be NIFTI (.nii) files
% Output is a structure with following fields:
%     Dice
%     Jaccard (Volume overlap VO)
%     PPV (positive predictive value)
%     TPR (sensitivity)
%     LTPR (lesion TPR based on lesion count)
%     LFPR (lesion FPR based on lesion count)
%     VD (absolute volume difference)
%     SD (average symmetric surface distance)
% All definitions are based on the following paper
% http://www.sciencedirect.com/science/article/pii/S1053811911003740

conn=18;
if ~ischar(seg) || ~ischar(truth)
    fprintf('File names must be character strings.\n');
    X=[];
    return;
end
if ~strcmpi(seg(end-2:end),'nii') ||  ~strcmpi(truth(end-2:end),'nii') 
    fprintf('Volumes must be NIFTI nii.\n');
    X=[];
    return;
end
temp=load_untouch_nii(seg);
res=prod(temp.hdr.dime.pixdim(2:4));
segvol=temp.img;
temp=load_untouch_nii(truth);
if res~=prod(temp.hdr.dime.pixdim(2:4))
    fprintf('Warning: Resolutions do not match. %.4f vs %.4f\n',res,prod(temp.hdr.dime.pixdim(2:4)))
end
truthvol=temp.img;
clear temp;
if length(unique(segvol))~=2 || length(unique(truthvol))~=2 
    fprintf('Volumes must be binary.\n');
    X=[];
    return;
end
if sum(size(segvol)==size(truthvol))~=3
    fprintf('Volumes must have same size, %dx%dx%d vs %dx%dx%d\n',size(segvol,1),....
        size(segvol,2),size(segvol,3),size(truthvol,1),size(truthvol,2),size(truthvol,3));
    X=[];
    return;
end

fprintf('Computing Dice,');
X.Dice=2*(length(find(segvol==1 & truthvol==1)))/(sum(segvol(:))+sum(truthvol(:)));

fprintf('Jaccard,');
X.Jaccard=(length(find(segvol==1 & truthvol==1)))/(length(find(segvol==1 | truthvol==1)));


fprintf('PPV,');
X.PPV=(length(find(segvol==1 & truthvol==1)))/...
   ( length(find(segvol==1 & truthvol==1)) + length(find(segvol==1 & truthvol==0)) ) ;

fprintf('Sensitivity,');
X.TPR=(length(find(segvol==1 & truthvol==1)))/(sum(truthvol(:)));

fprintf('Lesion FPR,');

cc=bwconncomp(segvol,conn);
indx=find(truthvol==1);
count=0;
for t=1:cc.NumObjects
     if isempty(intersect(cc.PixelIdxList{t},indx))
         count=count+1;
     end
end
X.LFPR=count/cc.NumObjects;
 
fprintf('Lesion TPR,');
cc=bwconncomp(truthvol,conn);
indx=find(segvol==1);
count=0;
for t=1:cc.NumObjects
    if ~isempty(intersect(cc.PixelIdxList{t},indx))       
        count=count+1;
    end
end
X.LTPR=count/cc.NumObjects;


fprintf('VD,');
X.VD=abs(sum(segvol(:))-sum(truthvol(:)))/sum(truthvol(:));


fprintf('SD\n');
A=[];B=[];
f=find(segvol(:)==1);
[x, y, z]=ind2sub(size(segvol),f);
A(1,:)=x;A(2,:)=y;A(3,:)=z;
f=find(truthvol(:)==1);
[x, y, z]=ind2sub(size(truthvol),f);
B(1,:)=x;B(2,:)=y;B(3,:)=z;


M=size(A,2);
fhd=zeros(1,M); 
parfor l=1:M
    tempdist=bsxfun(@minus,B,A(:,l));
    tempdist=sum((tempdist.^2),1);
    tempdist=sqrt(tempdist);
    indx=find(tempdist==min(tempdist));
    fhd(l) = tempdist(indx(1));
end
    
M=length(B);
rhd=zeros(1,M); 
parfor l=1:M
    tempdist=bsxfun(@minus,A,B(:,l));
    tempdist=sum((tempdist.^2),1);
    tempdist=sqrt(tempdist);
    indx=find(tempdist==min(tempdist));
    rhd(l) = tempdist(indx(1));
end
    
X.SD=res*(sum(fhd)+sum(rhd))/(length(find(segvol==1 | truthvol==1)));
X.ManualVol=res*sum(truthvol(:));
X.SegVol=res*sum(segvol(:));

